<?php
/**
 * Подключение файла конфигурации
 */
require_once __DIR__ . '/config.inc';

/**
 * RBS
 *
 * Интеграция платежного шлюза RBS с CMS интернет-магазинов
 *
 * @author RBS
 * @version 1.0
 */
class RBS {
    /**
     * АДРЕС ТЕСТОВОГО ШЛЮЗА
     *
     * @var string
     */
    const TEST_URL = TEST_URL;

    /**
     * АДРЕС БОЕВОГО ШЛЮЗА
     *
     * @var string
     */
    const PROD_URL = PROD_URL;

    /**
     * ЛОГИН МЕРЧАНТА
     *
     * @var string
     */
    private $user_name;

    /**
     * ПАРОЛЬ МЕРЧАНТА
     *
     * @var string
     */
    private $password;

    /**
     * ДВУХСТАДИЙНЫЙ ПЛАТЕЖ
     *
     * Если значение true - будет производиться двухстадийный платеж
     *
     * @var boolean
     */
    private $two_stage;

    /**
     * ТЕСТОВЫЙ РЕЖИМ
     *
     * Если значение true - плагин будет работать в тестовом режиме
     *
     * @var boolean
     */
    private $test_mode;

    /**
     * ЛОГИРОВАНИЕ
     *
     * Если значение true - плагин будет логировать запросы в ПШ
     *
     * @var boolean
     */
    private $logging;

    /**
     * КОНСТРУКТОР КЛАССА
     *
     * Заполнение свойств объекта
     *
     * @param string $user_name логин мерчанта
     * @param string $password пароль мерчанта
     * @param boolean $logging логирование
     * @param boolean $two_stage двухстадийный платеж
     * @param boolean $test_mode тестовый режим
     */
    public function RBS ($user_name, $password, $logging, $two_stage, $test_mode)
    {
        $this->user_name = $user_name;
        $this->password = $password;
        $this->logging = $logging;
        $this->two_stage = $two_stage;
        $this->test_mode = $test_mode;
    }

    /**
     * ЗАПРОС В ПШ
     *
     * Формирование запроса в платежный шлюз и парсинг JSON-ответа
     *
     * @param string $method метод запроса в ПШ
     * @param string[] $data данные в запросе
     * @return string[]
     */
    private function gateway($method, $data)
    {
        $data['userName'] = $this->user_name;
        $data['password'] = $this->password;
        if ($this->test_mode) {
            $url = self::TEST_URL;
        } else {
            $url = self::PROD_URL;
        }
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $url.$method,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($data)
        ));
        $response = curl_exec($curl);
        $response = json_decode($response, true);
        if ($this->logging) {
            $this->logger('Request: ' . $url . $method . ': ' . print_r($data, true).'Response: ' . print_r($response, true));
        }
        curl_close($curl);
        return $response;
    }

    /**
     * ЛОГГЕР
     *
     * Логирование запроса и ответа от ПШ
     *
     * @param string $var
     * @return integer
     */
    private function logger($var)
    {
        $date = '====== '.date('Y-m-d H:i:s')." =====\n";
        $result = $var;
        if (is_array($var) || is_object($var)) {
            $result = print_r($var, true);
        }
        $result .= "\n";
        return error_log($date.$result);
    }

    /**
     * MD ORDER ИЗ FORM URL
     *
     * @param string $url адрес платежной страницы
     * @return string
     */
    public function get_mdOrder_from_url($url)
    {
        $parse = parse_url($url);
        $mdOrder = explode('=', $parse['query']);
        return $mdOrder[1];
    }

    /**
     * РЕГИСТРАЦИЯ ЗАКАЗА
     *
     * Метод register.do или registerPreAuth.do
     *
     * @param string $order_number номер заказа в магазине
     * @param integer $amount сумма заказа
     * @param string $return_url страница, на которую необходимо вернуть пользователя
     * @return string[]
     */
    function register_order($order_number, $amount, $return_url)
    {
        $data = array(
            'orderNumber' => $order_number,
            'amount' => $amount,
            'returnUrl' => $return_url
        );
        if ($this->two_stage) { $method = 'registerPreAuth.do'; } else { $method = 'register.do'; }
        $response = $this->gateway($method, $data);
        return $response;
    }

    /**
     * СТАТУС ЗАКАЗА ПО ORDER ID
     *
     * Метод getOrderStatusExtended.do
     *
     * @param string $order_id номер заказа в магазине
     * @return string[]
     */
    public function get_order_status_by_orderId($order_id)
    {
        $data = array('orderId' => $order_id);
        $response = $this->gateway('getOrderStatusExtended.do', $data);
        return $response;
    }

    /**
     * СТАТУС ЗАКАЗА ПО ORDER NUMBER
     *
     * Метод getOrderStatusExtended.do
     *
     * @param string $order_number номер заказа в магазине
     * @return string[]
     */
    public function get_order_status_by_orderNumber($order_number)
    {
        $data = array('orderNumber' => $order_number);
        $response = $this->gateway('getOrderStatusExtended.do', $data);
        return $response;
    }
}